@@ -13,10 +13,11 @@ use yakv::storage::*;
 const RAND_SEED: u64 = 2021;
 const N_RECORDS_LARGE: usize = 1000000;
 const N_RECORDS_SMALL: usize = 10000;
+const TRANS_SIZE: usize = 10000;
 
 #[test]
 fn test_basic_ops() {
-    let store = open_store("test1.dbs", Some("test1.log"));
+    let store = open_store("test1.dbs", false);
     {
         let mut trans = store.start_transaction();
         trans.put(&v(b"1"), &v(b"one")).unwrap();
@@ -101,18 +102,27 @@ fn test_basic_ops() {
             .collect::<Vec<(Key, Value)>>(),
         [(v(b"2"), v(b"two-two")), (v(b"4"), v(b"four"))]
     );
+    drop(store);
+    let store = reopen_store("test1.dbs", false);
+    assert_eq!(
+        store
+            .range(v(b"2")..v(b"5"))
+            .flatten()
+            .collect::<Vec<(Key, Value)>>(),
+        [(v(b"2"), v(b"two-two")), (v(b"4"), v(b"four"))]
+    );
 }
 
 fn seq_benchmark(
     db_path: &str,
-    log_path: Option<&str>,
+    nosync: bool,
     n_records: usize,
     transaction_size: usize,
 ) -> Result<()> {
     let payload1: Vec<u8> = vec![1u8; 100];
     let payload2: Vec<u8> = vec![2u8; 100];
     {
-        let store = open_store(db_path, log_path);
+        let store = open_store(db_path, nosync);
         let mut key: u64 = 0;
 
         let mut now = Instant::now();
@@ -165,7 +175,7 @@ fn seq_benchmark(
     }
     {
         // reopen database
-        let store = reopen_store(db_path, log_path);
+        let store = reopen_store(db_path, nosync);
 
         let mut now = Instant::now();
         for i in 1..=n_records {
@@ -200,14 +210,14 @@ fn seq_benchmark(
 
 fn rnd_benchmark(
     db_path: &str,
-    log_path: Option<&str>,
+    nosync: bool,
     n_records: usize,
     transaction_size: usize,
 ) -> Result<()> {
     let payload1: Vec<u8> = vec![1u8; 100];
     let payload2: Vec<u8> = vec![2u8; 100];
     {
-        let store = open_store(db_path, log_path);
+        let store = open_store(db_path, nosync);
 
         let mut rand = StdRng::seed_from_u64(RAND_SEED);
         let mut now = Instant::now();
@@ -258,7 +268,7 @@ fn rnd_benchmark(
     }
     {
         // reopen database
-        let store = reopen_store(db_path, log_path);
+        let store = reopen_store(db_path, nosync);
 
         let mut now = Instant::now();
         let mut rand = StdRng::seed_from_u64(RAND_SEED);
@@ -290,48 +300,48 @@ fn rnd_benchmark(
 }
 
 #[test]
-fn seq_benchmark_wal_large_trans() {
-    assert!(seq_benchmark("test2.dbs", Some("test2.log"), N_RECORDS_LARGE, 1000,).is_ok());
+fn seq_benchmark_sync_large_trans() {
+    assert!(seq_benchmark("test2.dbs", false, N_RECORDS_LARGE, TRANS_SIZE,).is_ok());
 }
 
 #[test]
-fn seq_benchmark_wal_small_trans() {
-    assert!(seq_benchmark("test3.dbs", Some("test3.log"), N_RECORDS_SMALL, 1,).is_ok());
+fn seq_benchmark_sync_small_trans() {
+    assert!(seq_benchmark("test3.dbs", false, N_RECORDS_SMALL, 1,).is_ok());
 }
 
 #[test]
-fn seq_benchmark_nowal_large_trans() {
-    assert!(seq_benchmark("test4.dbs", None, N_RECORDS_LARGE, 1000,).is_ok());
+fn seq_benchmark_nosync_large_trans() {
+    assert!(seq_benchmark("test4.dbs", true, N_RECORDS_LARGE, TRANS_SIZE,).is_ok());
 }
 
 #[test]
-fn seq_benchmark_nowal_small_trans() {
-    assert!(seq_benchmark("test5.dbs", None, N_RECORDS_LARGE, 1,).is_ok());
+fn seq_benchmark_nosync_small_trans() {
+    assert!(seq_benchmark("test5.dbs", true, N_RECORDS_LARGE, 1,).is_ok());
 }
 
 #[test]
-fn rnd_benchmark_wal_large_trans() {
-    assert!(rnd_benchmark("test6.dbs", Some("test6.log"), N_RECORDS_LARGE, 1000,).is_ok());
+fn rnd_benchmark_sync_large_trans() {
+    assert!(rnd_benchmark("test6.dbs", false, N_RECORDS_LARGE, TRANS_SIZE,).is_ok());
 }
 
 #[test]
-fn rnd_benchmark_wal_small_trans() {
-    assert!(rnd_benchmark("test7.dbs", Some("test7.log"), N_RECORDS_SMALL, 1,).is_ok());
+fn rnd_benchmark_sync_small_trans() {
+    assert!(rnd_benchmark("test7.dbs", false, N_RECORDS_SMALL, 1,).is_ok());
 }
 
 #[test]
-fn rnd_benchmark_nowal_large_trans() {
-    assert!(rnd_benchmark("test8.dbs", None, N_RECORDS_LARGE, 1000,).is_ok());
+fn rnd_benchmark_nosync_large_trans() {
+    assert!(rnd_benchmark("test8.dbs", true, N_RECORDS_LARGE, TRANS_SIZE,).is_ok());
 }
 
 #[test]
-fn rnd_benchmark_nowal_small_trans() {
-    assert!(rnd_benchmark("test9.dbs", None, N_RECORDS_LARGE, 1,).is_ok());
+fn rnd_benchmark_nosync_small_trans() {
+    assert!(rnd_benchmark("test9.dbs", true, N_RECORDS_LARGE, 1,).is_ok());
 }
 
 #[test]
 fn test_acid() {
-    let store = open_store("test10.dbs", Some("test10.log"));
+    let store = open_store("test10.dbs", false);
 
     assert!(store
         .put_all(&mut (0..100).map(|key| {
@@ -383,15 +393,11 @@ fn test_acid() {
 
 #[test]
 fn test_recovery() {
-    let data_path = Path::new("test11.dbs");
-    let log_path = Path::new("test11.log");
+    let data_path = "test11.dbs";
     const N_KEYS: u64 = 100000;
     {
         let _ = std::fs::remove_file(&data_path);
-        let _ = std::fs::remove_file(&log_path);
-        let mut cfg = StorageConfig::default();
-        cfg.wal_flush_threshold = 1;
-        let store = Storage::open(data_path, Some(log_path), cfg).unwrap();
+        let store = open_store(data_path, false);
         {
             let mut trans = store.start_transaction();
             for key in 0..N_KEYS {
@@ -413,13 +419,9 @@ fn test_recovery() {
             }
             // transaction shoud be implicitly aborted
         }
-        store.shutdown().unwrap(); // do not truncate WAL
     }
     {
-        let store = Storage::open(data_path, Some(log_path), StorageConfig::default()).unwrap();
-        let recovery = store.get_recovery_status();
-        assert_eq!(recovery.recovered_transactions, 2);
-        assert!(recovery.wal_size > recovery.recovery_end);
+        let store = reopen_store(data_path, false);
         for key in 0..N_KEYS {
             assert_eq!(store.get(&pack(key)).unwrap().unwrap(), v(b"two"));
         }
@@ -443,7 +445,7 @@ fn do_selects(s: Arc<Storage>, n_records: usize) {
 
 #[test]
 fn test_parallel_access() {
-    let store = Arc::new(open_store("test1.dbs", None));
+    let store = Arc::new(open_store("test1.dbs", true));
     let n_writers = 10u32;
     let n_records = 10000u32;
     let mut threads = Vec::new();
@@ -489,18 +491,17 @@ fn pack(key: u64) -> Vec<u8> {
     key.to_be_bytes().to_vec()
 }
 
-fn open_store(data_file: &str, log_file: Option<&str>) -> Storage {
+fn open_store(data_file: &str, nosync: bool) -> Storage {
     let data_path = Path::new(data_file);
-    let log_path = log_file.map(|wal| Path::new(wal));
     let _ = std::fs::remove_file(&data_path);
-    if let Some(log) = log_path {
-        let _ = std::fs::remove_file(&log);
-    }
-    Storage::open(data_path, log_path, StorageConfig::default()).unwrap()
+    let mut cfg = StorageConfig::default();
+    cfg.nosync = nosync;
+    Storage::open(data_path, cfg).unwrap()
 }
 
-fn reopen_store(data_file: &str, log_file: Option<&str>) -> Storage {
+fn reopen_store(data_file: &str, nosync: bool) -> Storage {
     let data_path = Path::new(data_file);
-    let log_path = log_file.map(|wal| Path::new(wal));
-    Storage::open(data_path, log_path, StorageConfig::default()).unwrap()
+    let mut cfg = StorageConfig::default();
+    cfg.nosync = nosync;
+    Storage::open(data_path, cfg).unwrap()
 }
